﻿export function GlobalAnimationClock() {
    var instance = null;
    
    var subscribers = new Set();
    var isRunning = false;
    var lastFrameTime = 0;
    var phase = 'normal';
    var phaseDuration = 0;
    
    this.getInstance = function() {
        if (!instance) {
            instance = new GlobalAnimationClock();
        }
        return instance;
    };
    
    this.start = function() {
        if (isRunning) return;
        isRunning = true;
        lastFrameTime = performance.now();
        phase = 'normal';
        phaseDuration = 2500;
        tick();
    };
    
    this.stop = function() {
        isRunning = false;
        subscribers.clear();
    };
    
    this.subscribe = function(callback) {
        subscribers.add(callback);
    };
    
    this.unsubscribe = function(callback) {
        subscribers.delete(callback);
    };
    
    function tick() {
        if (!isRunning) return;

        var currentTime = performance.now();
        var deltaTime = currentTime - lastFrameTime;

        if (deltaTime >= phaseDuration) {
            updatePhase();
            lastFrameTime = currentTime;
            notifySubscribers();
        }

        requestAnimationFrame(tick);
    }
    
    function updatePhase() {
        switch (phase) {
            case 'normal':
                phase = 'grow';
                phaseDuration = 200; // Transition to skull
                break;
            case 'grow':
                phase = 'skull';
                phaseDuration = 1000; // Skull display time
                break;
            case 'skull':
                phase = 'shrink';
                phaseDuration = 200; // Transition back
                break;
            case 'shrink':
                phase = 'normal';
                phaseDuration = 2250; // Normal icon time
                break;
        }
    }
    
    function notifySubscribers() {
        subscribers.forEach(function(callback) {
            callback();
        });
    }
    
    this.getPhase = function() {
        return phase;
    };
}

export var animationClock = new GlobalAnimationClock().getInstance();